# coding: utf-8
from __future__ import print_function


class CoreService(object):
    """TODO"""
    CORE_SERVICE_NAME = '__core__'

    def getTags(self, remote_service):
        pass

    def addTag(self, tag, remote_service):
        pass

    def removeTag(self, tag, remote_service):
        pass

    def clearTags(self, remote_service):
        pass

    def containsTags(self, tags, remote_service):
        pass

    def getChannelId(self, remote_service):
        pass